@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -32,6 +32,8 @@ USA.
 ;;; compiler.
 
 (declare (usual-integrations))
+
+(load-option 'COMPRESS)                        ; XXX ugh
 
 (define (finish-cross-compilation:directory directory #!optional force?)
   (let ((force? (if (default-object? force?) #f force?)))
@@ -42,12 +44,64 @@ USA.
 				    (or (string=? ns ".")
 					(string=? ns ".."))))
 			     (loop pathname)))
+			((let ((t (pathname-type pathname)))
+			   (and (string? t)
+				(string=? t "fni")))
+			 (finish-cross-compilation:info-file pathname force?))
 			((let ((t (pathname-type pathname)))
 			   (and (string? t)
 				(string=? t "moc")))
 			 (finish-cross-compilation:file pathname force?))))
 		(directory-read (pathname-as-directory directory))))))
 
+(define (finish-cross-compilation:files directory #!optional force?)
+  (let ((force? (if (default-object? force?) #f force?)))
+    (let loop ((directory directory))
+      (for-each (lambda (pathname)
+                 (cond ((file-directory? pathname)
+                        (if (not (let ((ns (file-namestring pathname)))
+                                   (or (string=? ns ".")
+                                       (string=? ns ".."))))
+                            (loop pathname)))
+                       ((let ((t (pathname-type pathname)))
+                          (and (string? t)
+                               (string=? t "moc")))
+                        (finish-cross-compilation:file pathname force?))))
+               (directory-read (pathname-as-directory directory))))))
+
+(define (finish-cross-compilation:info-files directory #!optional force?)
+  (let ((force? (if (default-object? force?) #f force?)))
+    (let loop ((directory directory))
+      (for-each (lambda (pathname)
+		  (cond ((file-directory? pathname)
+			 (if (not (let ((ns (file-namestring pathname)))
+				    (or (string=? ns ".")
+					(string=? ns ".."))))
+			     (loop pathname)))
+			((let ((t (pathname-type pathname)))
+			   (and (string? t)
+				(string=? t "fni")))
+			 (finish-cross-compilation:info-file pathname force?))))
+		(directory-read (pathname-as-directory directory))))))
+
+(define (finish-cross-compilation:info-file pathname #!optional force?)
+  (let* ((input-file (pathname-default-type pathname "fni"))
+	 (output-file (pathname-new-type input-file "bci")))
+    (if (or (if (default-object? force?) #t force?)
+	    (file-modification-time<? output-file input-file))
+	(with-notification
+	    (lambda (port)
+	      (write-string "Compressing info: " port)
+	      (write (enough-namestring input-file) port)
+	      (write-string " => " port)
+	      (write (enough-namestring output-file) port))
+	    (lambda ()
+	      (let ((inf (fasload input-file #t)))
+		(call-with-temporary-filename
+		  (lambda (temp)
+		    (fasdump inf temp #t)
+		    (compress temp output-file)))))))))
+
 (define (finish-cross-compilation:file input-file #!optional force?)
   (let* ((input-file (pathname-default-type input-file "moc"))
 	 (output-file (pathname-new-type input-file "com")))
@@ -71,17 +125,20 @@ USA.
     (if (null? others)
 	expression
 	(scode/make-comment
-	 (make-dbg-info-vector
-	  (let ((all-blocks
-		 (list->vector
-		  (cons
-		   (compiled-code-address->block expression)
-		   others))))
-	    (if compile-by-procedures?
-		(list 'COMPILED-BY-PROCEDURES
-		      all-blocks
-		      (list->vector others))
-		all-blocks)))
+	 ;; Keep in sync with "toplev.scm" and with "runtime/infstr.scm".
+	 (vector
+	  '|#[(runtime compiler-info)dbg-info-vector]|
+	  (if compile-by-procedures?
+	      'compiled-by-procedures
+	      'compiled-as-unit)
+	  (compiled-code-address->block expression)
+	  (list->vector
+	   (map (lambda (other)
+		  (vector-ref other 2))
+		others))
+	  '()
+	  '()
+	  '())
 	 expression))))
 
 (define (cross-link-end object)
@@ -90,7 +147,7 @@ USA.
      (if (compiled-code-block? code-vector)
 	 code-vector
 	 (begin
-	   (guarantee-vector code-vector #f)
+	   (guarantee vector? code-vector #f)
 	   (let ((new-code-vector
 		  (cross-link/finish-assembly
 		   (cc-code-block/bit-string code-vector)
