@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -31,7 +31,7 @@ USA.
 (define-syntax define-trivial-instruction
   (sc-macro-transformer
    (lambda (form environment)
-     (if (syntax-match? '(IDENTIFIER DATUM * DATUM) (cdr form))
+     (if (syntax-match? '(identifier datum * datum) (cdr form))
 	 `(DEFINE-INSTRUCTION ,(cadr form)
 	    (()
 	     (BITS (8 ,(close-syntax (caddr form) environment)))
@@ -75,7 +75,7 @@ USA.
   (sc-macro-transformer
    (lambda (form environment)
      environment
-     (if (syntax-match? '(IDENTIFIER * SYMBOL) (cdr form))
+     (if (syntax-match? '(identifier * symbol) (cdr form))
 	 `(DEFINE (,(cadr form) EXPRESSION)
 	    (LET ((MATCH-RESULT (PATTERN-LOOKUP ,ea-database-name EXPRESSION)))
 	      (AND MATCH-RESULT
