@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -44,7 +44,7 @@ USA.
 
 (define (clear-abbrev-table table)
   (set! abbrevs-changed? #t)
-  (hash-table/clear! table))
+  (hash-table-clear! table))
 
 (define (define-abbrev table abbrev expansion #!optional hook count)
   (let ((hook (if (default-object? hook) #f hook))
@@ -52,10 +52,10 @@ USA.
     (guarantee-abbrev-table table 'DEFINE-ABBREV)
     (guarantee-string abbrev 'DEFINE-ABBREV)
     (guarantee-string expansion 'DEFINE-ABBREV)
-    (if hook (guarantee-symbol hook 'DEFINE-ABBREV))
-    (guarantee-exact-nonnegative-integer count 'DEFINE-ABBREV)
+    (if hook (guarantee symbol? hook 'DEFINE-ABBREV))
+    (guarantee exact-nonnegative-integer? count 'DEFINE-ABBREV)
     (set! abbrevs-changed? #t)
-    (hash-table/put! table
+    (hash-table-set! table
 		     (string-downcase abbrev)
 		     (make-abbrev-entry expansion hook count))))
 
@@ -72,25 +72,25 @@ USA.
   (guarantee-abbrev-table table 'UNDEFINE-ABBREV)
   (guarantee-string abbrev 'UNDEFINE-ABBREV)
   (set! abbrevs-changed? #t)
-  (hash-table/remove! table (string-downcase abbrev)))
+  (hash-table-delete! table (string-downcase abbrev)))
 
 (define (abbrev-entry abbrev where)
   (let ((abbrev
 	 (string-downcase
 	  (cond ((string? abbrev) abbrev)
-		((symbol? abbrev) (symbol-name abbrev))
+		((symbol? abbrev) (symbol->string abbrev))
 		(else
 		 (error:wrong-type-argument abbrev "string"
 					    'ABBREV-EXPANSION))))))
     (if (abbrev-table? where)
-	(hash-table/get where abbrev #f)
+	(hash-table-ref/default where abbrev #f)
 	(let ((buffer (if (not where) (selected-buffer) where)))
 	  (or (let ((table (ref-variable local-abbrev-table buffer)))
 		(and table
-		     (hash-table/get table abbrev #f)))
-	      (hash-table/get (ref-variable global-abbrev-table #f)
-			      abbrev
-			      #f))))))
+		     (hash-table-ref/default table abbrev #f)))
+	      (hash-table-ref/default (ref-variable global-abbrev-table #f)
+				      abbrev
+				      #f))))))
 
 (define (abbrev-expansion abbrev where)
   (let ((entry (abbrev-entry abbrev where)))
@@ -472,9 +472,9 @@ Mark is set after the inserted text."
      (lambda (name)
        (let ((table (get-named-abbrev-table name)))
 	 (insert-string "(" mark)
-	 (insert-string (symbol-name name) mark)
+	 (insert-string (symbol->string name) mark)
 	 (insert-string ")\n\n" mark)
-	 (hash-table/for-each table
+	 (hash-table-walk table
 	   (lambda (abbrev entry)
 	     (if (abbrev-entry-expansion entry)
 		 (begin
@@ -601,7 +601,7 @@ The argument FILENAME is the file name to write."
 	       (write name port)
 	       (write-string " '(" port)
 	       (newline port)
-	       (hash-table/for-each table
+	       (hash-table-walk table
 		 (lambda (abbrev entry)
 		   (if (abbrev-entry-expansion entry)
 		       (begin
