@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -478,10 +478,10 @@ This is a good thing to set in mode hooks."
   (list comint-dynamic-complete-filename)
   (lambda (object)
     (and (list? object)
-	 (for-all? object
-	   (lambda (object)
-	     (and (procedure? object)
-		  (procedure-arity-valid? object 0)))))))
+	 (every (lambda (object)
+		  (and (procedure? object)
+		       (procedure-arity-valid? object 0)))
+		object))))
 
 (define-command comint-dynamic-complete
   "Dynamically perform completion at point.
