@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -102,10 +102,10 @@ USA.
   (let ((table (make-string-hash-table)))
     (for-each (lambda (delta)
 		(let ((key (vector-ref delta 0)))
-		  (let ((entry (hash-table/get table key #f)))
+		  (let ((entry (hash-table-ref/default table key #f)))
 		    (if entry
 			(error "duplicate delta entry" delta entry)))
-		  (hash-table/put! table key
+		  (hash-table-set! table key
 				   (make-rcs-delta key
 						   (vector-ref delta 1)
 						   (vector-ref delta 2)
@@ -115,11 +115,11 @@ USA.
 	      deltas)
     (let ((num->delta
 	   (lambda (key)
-	     (let ((delta (hash-table/get table key #f)))
+	     (let ((delta (hash-table-ref/default table key #f)))
 	       (if (not delta)
 		   (error "unknown delta number" key))
 	       delta))))
-      (hash-table/for-each table
+      (hash-table-walk table
 	(lambda (key delta)
 	  key
 	  (do ((branches (rcs-delta/branches delta) (cdr branches)))
