@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -253,7 +253,7 @@ abort_to_interpreter_argument (void)
 long prim_apply_error_code;
 
 void
-Interpret (int pop_return_p)
+Interpret (void)
 {
   long dispatch_code;
   struct interpreter_state_s new_state;
@@ -274,10 +274,7 @@ Interpret (int pop_return_p)
   switch (dispatch_code)
     {
     case 0:			/* first time */
-      if (pop_return_p)
-	goto pop_return;	/* continue */
-      else
-	break;			/* fall into eval */
+      break;			/* fall into eval */
 
     case PRIM_APPLY:
       PROCEED_AFTER_PRIMITIVE ();
@@ -407,30 +404,33 @@ Interpret (int pop_return_p)
     {
     case TC_BIG_FIXNUM:         /* The self evaluating items */
     case TC_BIG_FLONUM:
-    case TC_CHARACTER_STRING:
+    case TC_BYTEVECTOR:
     case TC_CHARACTER:
+    case TC_CHARACTER_STRING:
     case TC_COMPILED_CODE_BLOCK:
     case TC_COMPLEX:
+    case TC_CONSTANT:
     case TC_CONTROL_POINT:
     case TC_DELAYED:
     case TC_ENTITY:
     case TC_ENVIRONMENT:
     case TC_EXTENDED_PROCEDURE:
+    case TC_FALSE:
     case TC_FIXNUM:
     case TC_HUNK3_A:
     case TC_HUNK3_B:
     case TC_INTERNED_SYMBOL:
     case TC_LIST:
     case TC_NON_MARKED_VECTOR:
-    case TC_NULL:
     case TC_PRIMITIVE:
     case TC_PROCEDURE:
     case TC_QUAD:
     case TC_RATNUM:
+    case TC_RECORD:
     case TC_REFERENCE_TRAP:
     case TC_RETURN_CODE:
+    case TC_UNICODE_STRING:
     case TC_UNINTERNED_SYMBOL:
-    case TC_CONSTANT:
     case TC_VECTOR:
     case TC_VECTOR_16B:
     case TC_VECTOR_1B:
@@ -467,16 +467,6 @@ Interpret (int pop_return_p)
 	DO_NTH_THEN (RC_COMB_SAVE_VALUE, (length + 1));
       }
 
-    case TC_COMBINATION_1:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 1);
-      PUSH_ENV ();
-      DO_NTH_THEN (RC_COMB_1_PROCEDURE, COMB_1_ARG_1);
-
-    case TC_COMBINATION_2:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 2);
-      PUSH_ENV ();
-      DO_NTH_THEN (RC_COMB_2_FIRST_OPERAND, COMB_2_ARG_2);
-
     case TC_COMMENT:
       REDUCES_TO_NTH (COMMENT_EXPRESSION);
 
@@ -517,10 +507,6 @@ Interpret (int pop_return_p)
       Free += 2;
       break;
 
-    case TC_IN_PACKAGE:
-      Will_Push (CONTINUATION_SIZE);
-      PUSH_NTH_THEN (RC_EXECUTE_IN_PACKAGE_CONTINUE, IN_PACKAGE_ENVIRONMENT);
-
     case TC_LAMBDA:
     case TC_LEXPR:
       /* Deliberately omitted: EVAL_GC_CHECK (2); */
@@ -533,41 +519,17 @@ Interpret (int pop_return_p)
     case TC_MANIFEST_NM_VECTOR:
       EVAL_ERROR (ERR_EXECUTE_MANIFEST_VECTOR);
 
-    case TC_PCOMB0:
-      /* The argument to Will_Eventually_Push is determined by how
-	 much will be on the stack if we back out of the primitive.  */
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      Finished_Eventual_Pushing (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      SET_EXP (OBJECT_NEW_TYPE (TC_PRIMITIVE, GET_EXP));
-      goto primitive_internal_apply;
-
-    case TC_PCOMB1:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 1);
-      DO_NTH_THEN (RC_PCOMB1_APPLY, PCOMB1_ARG_SLOT);
-
-    case TC_PCOMB2:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 2);
-      PUSH_ENV ();
-      DO_NTH_THEN (RC_PCOMB2_DO_1, PCOMB2_ARG_2_SLOT);
-
-    case TC_PCOMB3:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 3);
-      PUSH_ENV ();
-      DO_NTH_THEN (RC_PCOMB3_DO_2, PCOMB3_ARG_3_SLOT);
-
     case TC_SCODE_QUOTE:
       SET_VAL (MEMORY_REF (GET_EXP, SCODE_QUOTE_OBJECT));
       break;
 
-    case TC_SEQUENCE_2:
+    case TC_SEQUENCE:
       Will_Push (CONTINUATION_SIZE + 1);
       PUSH_ENV ();
-      PUSH_NTH_THEN (RC_SEQ_2_DO_2, SEQUENCE_1);
+      PUSH_NTH_THEN (RC_EXECUTE_SEQUENCE_FINISH, SEQUENCE_1);
 
-    case TC_SEQUENCE_3:
-      Will_Push (CONTINUATION_SIZE + 1);
-      PUSH_ENV ();
-      PUSH_NTH_THEN (RC_SEQ_3_DO_2, SEQUENCE_1);
+    case TC_SYNTAX_ERROR:
+      EVAL_ERROR (ERR_SYNTAX_ERROR);
 
     case TC_THE_ENVIRONMENT:
       SET_VAL (GET_ENV);
@@ -633,27 +595,6 @@ Interpret (int pop_return_p)
 
   switch (OBJECT_DATUM (GET_RET))
     {
-    case RC_COMB_1_PROCEDURE:
-      POP_ENV ();
-      PUSH_VAL ();		/* Arg. 1 */
-      STACK_PUSH (SHARP_F);	/* Operator */
-      PUSH_APPLY_FRAME_HEADER (1);
-      Finished_Eventual_Pushing (CONTINUATION_SIZE);
-      DO_ANOTHER_THEN (RC_COMB_APPLY_FUNCTION, COMB_1_FN);
-
-    case RC_COMB_2_FIRST_OPERAND:
-      POP_ENV ();
-      PUSH_VAL ();
-      PUSH_ENV ();
-      DO_ANOTHER_THEN (RC_COMB_2_PROCEDURE, COMB_2_ARG_1);
-
-    case RC_COMB_2_PROCEDURE:
-      POP_ENV ();
-      PUSH_VAL ();		/* Arg 1, just calculated */
-      STACK_PUSH (SHARP_F);	/* Function */
-      PUSH_APPLY_FRAME_HEADER (2);
-      Finished_Eventual_Pushing (CONTINUATION_SIZE);
-      DO_ANOTHER_THEN (RC_COMB_APPLY_FUNCTION, COMB_2_FN);
 
     case RC_COMB_APPLY_FUNCTION:
       END_SUBPROBLEM ();
@@ -839,15 +780,6 @@ Interpret (int pop_return_p)
         POP_RETURN_ERROR (result);
       }
 
-    case RC_EXECUTE_IN_PACKAGE_CONTINUE:
-      if (ENVIRONMENT_P (GET_VAL))
-	{
-	  END_SUBPROBLEM ();
-	  SET_ENV (GET_VAL);
-	  REDUCES_TO_NTH (IN_PACKAGE_EXPRESSION);
-	}
-      POP_RETURN_ERROR (ERR_BAD_FRAME);
-
     case RC_HALT:
       Microcode_Termination (TERM_TERM_HANDLER);
 
@@ -893,6 +825,7 @@ Interpret (int pop_return_p)
     internal_apply_val:
 
       (APPLY_FRAME_PROCEDURE ()) = GET_VAL;
+      /* fall through */
 
     case RC_INTERNAL_APPLY:
     internal_apply:
@@ -944,6 +877,7 @@ Interpret (int pop_return_p)
 
 	      (STACK_REF (0)) = (MEMORY_REF (Function, ENTITY_OPERATOR));
 	      PUSH_APPLY_FRAME_HEADER (frame_size);
+	    entity_apply:
 	      /* This must be done to prevent an infinite push loop by
 		 an entity whose handler is the entity itself or some
 		 other such loop.  Of course, it will die if stack overflow
@@ -952,194 +886,201 @@ Interpret (int pop_return_p)
 	      goto internal_apply;
 	    }
 
-	  case TC_PROCEDURE:
+	  case TC_RECORD:
 	    {
+	      SCHEME_OBJECT applicator = record_applicator(Function);
+	      if (applicator == SHARP_F)
+		APPLICATION_ERROR (ERR_INAPPLICABLE_OBJECT);
 	      unsigned long frame_size = (APPLY_FRAME_SIZE ());
-	      Function = (MEMORY_REF (Function, PROCEDURE_LAMBDA_EXPR));
-	      {
-		SCHEME_OBJECT formals
-		  = (MEMORY_REF (Function, LAMBDA_FORMALS));
-
-		if ((frame_size != (VECTOR_LENGTH (formals)))
-		    && (((OBJECT_TYPE (Function)) != TC_LEXPR)
-			|| (frame_size < (VECTOR_LENGTH (formals)))))
-		  APPLICATION_ERROR (ERR_WRONG_NUMBER_OF_ARGUMENTS);
-	      }
-	      if (GC_NEEDED_P (frame_size + 1))
-		{
-		  PREPARE_APPLY_INTERRUPT ();
-		  IMMEDIATE_GC (frame_size + 1);
-		}
-	      {
-		SCHEME_OBJECT * end = (Free + 1 + frame_size);
-		SCHEME_OBJECT env
-		  = (MAKE_POINTER_OBJECT (TC_ENVIRONMENT, Free));
-		(*Free++) = (MAKE_OBJECT (TC_MANIFEST_VECTOR, frame_size));
-		(void) STACK_POP ();
-		while (Free < end)
-		  (*Free++) = (STACK_POP ());
-		SET_ENV (env);
-		REDUCES_TO (MEMORY_REF (Function, LAMBDA_SCODE));
-	      }
+	      (STACK_REF (0)) = applicator;
+	      PUSH_APPLY_FRAME_HEADER (frame_size);
+	      goto entity_apply;
 	    }
 
-	  case TC_CONTROL_POINT:
-	    if ((APPLY_FRAME_SIZE ()) != 2)
-	      APPLICATION_ERROR (ERR_WRONG_NUMBER_OF_ARGUMENTS);
-	    SET_VAL (* (APPLY_FRAME_ARGS ()));
-	    unpack_control_point (Function);
-	    RESET_HISTORY ();
-	    goto pop_return;
-
-	    /* After checking the number of arguments, remove the
-	       frame header since primitives do not expect it.
-
-	       NOTE: This code must match the application code which
-	       follows primitive_internal_apply.  */
-
-	  case TC_PRIMITIVE:
-	    if (!IMPLEMENTED_PRIMITIVE_P (Function))
-	      APPLICATION_ERROR (ERR_UNIMPLEMENTED_PRIMITIVE);
-	    {
-	      unsigned long n_args = (APPLY_FRAME_N_ARGS ());
-
-
-	      /* Note that the first test below will fail for lexpr
-		 primitives.  */
-
-	      if (n_args != (PRIMITIVE_ARITY (Function)))
+	    case TC_PROCEDURE:
+	      {
+		unsigned long frame_size = (APPLY_FRAME_SIZE ());
+		Function = (MEMORY_REF (Function, PROCEDURE_LAMBDA_EXPR));
 		{
-		  if ((PRIMITIVE_ARITY (Function)) != LEXPR_PRIMITIVE_ARITY)
+		  SCHEME_OBJECT formals
+		    = (MEMORY_REF (Function, LAMBDA_FORMALS));
+
+		  if ((frame_size != (VECTOR_LENGTH (formals)))
+		      && (((OBJECT_TYPE (Function)) != TC_LEXPR)
+			  || (frame_size < (VECTOR_LENGTH (formals)))))
 		    APPLICATION_ERROR (ERR_WRONG_NUMBER_OF_ARGUMENTS);
-		  SET_LEXPR_ACTUALS (n_args);
 		}
-	      stack_pointer = (APPLY_FRAME_ARGS ());
-	      SET_EXP (Function);
-	      APPLY_PRIMITIVE_FROM_INTERPRETER (Function);
-	      POP_PRIMITIVE_FRAME (n_args);
+		if (GC_NEEDED_P (frame_size + 1))
+		  {
+		    PREPARE_APPLY_INTERRUPT ();
+		    IMMEDIATE_GC (frame_size + 1);
+		  }
+		{
+		  SCHEME_OBJECT * end = (Free + 1 + frame_size);
+		  SCHEME_OBJECT env
+		    = (MAKE_POINTER_OBJECT (TC_ENVIRONMENT, Free));
+		  (*Free++) = (MAKE_OBJECT (TC_MANIFEST_VECTOR, frame_size));
+		  (void) STACK_POP ();
+		  while (Free < end)
+		    (*Free++) = (STACK_POP ());
+		  SET_ENV (env);
+		  REDUCES_TO (MEMORY_REF (Function, LAMBDA_SCODE));
+		}
+	      }
+
+	    case TC_CONTROL_POINT:
+	      if ((APPLY_FRAME_SIZE ()) != 2)
+		APPLICATION_ERROR (ERR_WRONG_NUMBER_OF_ARGUMENTS);
+	      SET_VAL (* (APPLY_FRAME_ARGS ()));
+	      unpack_control_point (Function);
+	      RESET_HISTORY ();
 	      goto pop_return;
-	    }
 
-	  case TC_EXTENDED_PROCEDURE:
-	    {
-	      SCHEME_OBJECT lambda;
-	      SCHEME_OBJECT temp;
-	      unsigned long nargs;
-	      unsigned long nparams;
-	      unsigned long formals;
-	      unsigned long params;
-	      unsigned long auxes;
-	      long rest_flag;
-	      long size;
-	      long i;
-	      SCHEME_OBJECT * scan;
+	      /* After checking the number of arguments, remove the
+		 frame header since primitives do not expect it. */
 
-	      nargs = (POP_APPLY_FRAME_HEADER ());
-	      lambda = (MEMORY_REF (Function, PROCEDURE_LAMBDA_EXPR));
-	      Function = (MEMORY_REF (lambda, ELAMBDA_NAMES));
-	      nparams = ((VECTOR_LENGTH (Function)) - 1);
-	      Function = (Get_Count_Elambda (lambda));
-	      formals = (Elambda_Formals_Count (Function));
-	      params = ((Elambda_Opts_Count (Function)) + formals);
-	      rest_flag = (Elambda_Rest_Flag (Function));
-	      auxes = (nparams - (params + rest_flag));
+	    case TC_PRIMITIVE:
+	      if (!IMPLEMENTED_PRIMITIVE_P (Function))
+		APPLICATION_ERROR (ERR_UNIMPLEMENTED_PRIMITIVE);
+	      {
+		unsigned long n_args = (APPLY_FRAME_N_ARGS ());
 
-	      if ((nargs < formals) || (!rest_flag && (nargs > params)))
-		{
-		  PUSH_APPLY_FRAME_HEADER (nargs);
-		  APPLICATION_ERROR (ERR_WRONG_NUMBER_OF_ARGUMENTS);
-		}
-	      /* size includes the procedure slot, but not the header.  */
-	      size = (params + rest_flag + auxes + 1);
-	      if (GC_NEEDED_P
-		  (size + 1
-		   + ((nargs > params)
-		      ? (2 * (nargs - params))
-		      : 0)))
-		{
-		  PUSH_APPLY_FRAME_HEADER (nargs);
-		  PREPARE_APPLY_INTERRUPT ();
-		  IMMEDIATE_GC
+		/* Note that the first test below will fail for lexpr
+		   primitives.  */
+
+		if (n_args != (PRIMITIVE_ARITY (Function)))
+		  {
+		    if ((PRIMITIVE_ARITY (Function)) != LEXPR_PRIMITIVE_ARITY)
+		      APPLICATION_ERROR (ERR_WRONG_NUMBER_OF_ARGUMENTS);
+		    SET_LEXPR_ACTUALS (n_args);
+		  }
+		stack_pointer = (APPLY_FRAME_ARGS ());
+		SET_EXP (Function);
+		APPLY_PRIMITIVE_FROM_INTERPRETER (Function);
+		POP_PRIMITIVE_FRAME (n_args);
+		goto pop_return;
+	      }
+
+	    case TC_EXTENDED_PROCEDURE:
+	      {
+		SCHEME_OBJECT lambda;
+		SCHEME_OBJECT temp;
+		unsigned long nargs;
+		unsigned long nparams;
+		unsigned long formals;
+		unsigned long params;
+		unsigned long auxes;
+		long rest_flag;
+		long size;
+		long i;
+		SCHEME_OBJECT * scan;
+
+		nargs = (POP_APPLY_FRAME_HEADER ());
+		lambda = (MEMORY_REF (Function, PROCEDURE_LAMBDA_EXPR));
+		Function = (MEMORY_REF (lambda, ELAMBDA_NAMES));
+		nparams = ((VECTOR_LENGTH (Function)) - 1);
+		Function = (Get_Count_Elambda (lambda));
+		formals = (Elambda_Formals_Count (Function));
+		params = ((Elambda_Opts_Count (Function)) + formals);
+		rest_flag = (Elambda_Rest_Flag (Function));
+		auxes = (nparams - (params + rest_flag));
+
+		if ((nargs < formals) || (!rest_flag && (nargs > params)))
+		  {
+		    PUSH_APPLY_FRAME_HEADER (nargs);
+		    APPLICATION_ERROR (ERR_WRONG_NUMBER_OF_ARGUMENTS);
+		  }
+		/* size includes the procedure slot, but not the header.  */
+		size = (params + rest_flag + auxes + 1);
+		if (GC_NEEDED_P
 		    (size + 1
 		     + ((nargs > params)
 			? (2 * (nargs - params))
-			: 0));
-		}
-	      scan = Free;
-	      temp = (MAKE_POINTER_OBJECT (TC_ENVIRONMENT, scan));
-	      (*scan++) = (MAKE_OBJECT (TC_MANIFEST_VECTOR, size));
-	      if (nargs <= params)
-		{
-		  for (i = (nargs + 1); (--i) >= 0; )
-		    (*scan++) = (STACK_POP ());
-		  for (i = (params - nargs); (--i) >= 0; )
-		    (*scan++) = DEFAULT_OBJECT;
-		  if (rest_flag)
-		    (*scan++) = EMPTY_LIST;
-		  for (i = auxes; (--i) >= 0; )
-		    (*scan++) = UNASSIGNED_OBJECT;
-		}
-	      else
-		{
-		  /* rest_flag must be true. */
-		  SCHEME_OBJECT list
-		    = (MAKE_POINTER_OBJECT (TC_LIST, (scan + size)));
-		  for (i = (params + 1); (--i) >= 0; )
-		    (*scan++) = (STACK_POP ());
-		  (*scan++) = list;
-		  for (i = auxes; (--i) >= 0; )
-		    (*scan++) = UNASSIGNED_OBJECT;
-		  /* Now scan == OBJECT_ADDRESS (list) */
-		  for (i = (nargs - params); (--i) >= 0; )
-		    {
+			: 0)))
+		  {
+		    PUSH_APPLY_FRAME_HEADER (nargs);
+		    PREPARE_APPLY_INTERRUPT ();
+		    IMMEDIATE_GC
+		      (size + 1
+		       + ((nargs > params)
+			  ? (2 * (nargs - params))
+			  : 0));
+		  }
+		scan = Free;
+		temp = (MAKE_POINTER_OBJECT (TC_ENVIRONMENT, scan));
+		(*scan++) = (MAKE_OBJECT (TC_MANIFEST_VECTOR, size));
+		if (nargs <= params)
+		  {
+		    for (i = (nargs + 1); (--i) >= 0; )
 		      (*scan++) = (STACK_POP ());
-		      (*scan) = MAKE_POINTER_OBJECT (TC_LIST, (scan + 1));
-		      scan += 1;
-		    }
-		  (scan[-1]) = EMPTY_LIST;
-		}
+		    for (i = (params - nargs); (--i) >= 0; )
+		      (*scan++) = DEFAULT_OBJECT;
+		    if (rest_flag)
+		      (*scan++) = EMPTY_LIST;
+		    for (i = auxes; (--i) >= 0; )
+		      (*scan++) = UNASSIGNED_OBJECT;
+		  }
+		else
+		  {
+		    /* rest_flag must be true. */
+		    SCHEME_OBJECT list
+		      = (MAKE_POINTER_OBJECT (TC_LIST, (scan + size)));
+		    for (i = (params + 1); (--i) >= 0; )
+		      (*scan++) = (STACK_POP ());
+		    (*scan++) = list;
+		    for (i = auxes; (--i) >= 0; )
+		      (*scan++) = UNASSIGNED_OBJECT;
+		    /* Now scan == OBJECT_ADDRESS (list) */
+		    for (i = (nargs - params); (--i) >= 0; )
+		      {
+			(*scan++) = (STACK_POP ());
+			(*scan) = MAKE_POINTER_OBJECT (TC_LIST, (scan + 1));
+			scan += 1;
+		      }
+		    (scan[-1]) = EMPTY_LIST;
+		  }
 
-	      Free = scan;
-	      SET_ENV (temp);
-	      REDUCES_TO (Get_Body_Elambda (lambda));
-	    }
+		Free = scan;
+		SET_ENV (temp);
+		REDUCES_TO (Get_Body_Elambda (lambda));
+	      }
 
 #ifdef CC_SUPPORT_P
-	  case TC_COMPILED_ENTRY:
-	    {
-	      guarantee_cc_return (1 + (APPLY_FRAME_SIZE ()));
-	      dispatch_code = (apply_compiled_procedure ());
+	    case TC_COMPILED_ENTRY:
+	      {
+		guarantee_cc_return (1 + (APPLY_FRAME_SIZE ()));
+		dispatch_code = (apply_compiled_procedure ());
 
-	    return_from_compiled_code:
-	      switch (dispatch_code)
-		{
-		case PRIM_DONE:
-		  goto pop_return;
+	      return_from_compiled_code:
+		switch (dispatch_code)
+		  {
+		  case PRIM_DONE:
+		    goto pop_return;
 
-		case PRIM_APPLY:
-		  goto internal_apply;
+		  case PRIM_APPLY:
+		    goto internal_apply;
 
-		case PRIM_INTERRUPT:
-		  SIGNAL_INTERRUPT (PENDING_INTERRUPTS ());
+		  case PRIM_INTERRUPT:
+		    SIGNAL_INTERRUPT (PENDING_INTERRUPTS ());
 
-		case PRIM_APPLY_INTERRUPT:
-		  PREPARE_APPLY_INTERRUPT ();
-		  SIGNAL_INTERRUPT (PENDING_INTERRUPTS ());
+		  case PRIM_APPLY_INTERRUPT:
+		    PREPARE_APPLY_INTERRUPT ();
+		    SIGNAL_INTERRUPT (PENDING_INTERRUPTS ());
 
-		case ERR_INAPPLICABLE_OBJECT:
-		case ERR_WRONG_NUMBER_OF_ARGUMENTS:
-		  APPLICATION_ERROR (dispatch_code);
+		  case ERR_INAPPLICABLE_OBJECT:
+		  case ERR_WRONG_NUMBER_OF_ARGUMENTS:
+		    APPLICATION_ERROR (dispatch_code);
 
-		default:
-		  Do_Micro_Error (dispatch_code, true);
-		  goto internal_apply;
-		}
-	    }
+		  default:
+		    Do_Micro_Error (dispatch_code, true);
+		    goto internal_apply;
+		  }
+	      }
 #endif
 
-	  default:
-	    APPLICATION_ERROR (ERR_INAPPLICABLE_OBJECT);
-	  }
+	    default:
+	      APPLICATION_ERROR (ERR_INAPPLICABLE_OBJECT);
+	    }
       }
 
     case RC_JOIN_STACKLETS:
@@ -1155,76 +1096,6 @@ Interpret (int pop_return_p)
       EXIT_CRITICAL_SECTION ({ SAVE_CONT (); });
       break;
 
-    case RC_PCOMB1_APPLY:
-      END_SUBPROBLEM ();
-      PUSH_VAL ();		/* Argument value */
-      Finished_Eventual_Pushing (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      SET_EXP (MEMORY_REF (GET_EXP, PCOMB1_FN_SLOT));
-
-    primitive_internal_apply:
-
-#ifdef COMPILE_STEPPER
-      if (trapping
-	  && (!WITHIN_CRITICAL_SECTION_P ())
-	  && ((FETCH_APPLY_TRAPPER ()) != SHARP_F))
-	{
-	  Will_Push (3);
-	  PUSH_EXP ();
-	  STACK_PUSH (FETCH_APPLY_TRAPPER ());
-	  PUSH_APPLY_FRAME_HEADER (1 + (PRIMITIVE_N_PARAMETERS (GET_EXP)));
-	  Pushed ();
-	  trapping = false;
-	  goto Apply_Non_Trapping;
-	}
-#endif /* COMPILE_STEPPER */
-
-      /* NOTE: This code must match the code in the TC_PRIMITIVE
-	 case of internal_apply.
-	 This code is simpler because:
-	 1) The arity was checked at syntax time.
-	 2) We don't have to deal with "lexpr" primitives.
-	 3) We don't need to worry about unimplemented primitives because
-	 unimplemented primitives will cause an error at invocation.  */
-      {
-	SCHEME_OBJECT primitive = GET_EXP;
-	APPLY_PRIMITIVE_FROM_INTERPRETER (primitive);
-	POP_PRIMITIVE_FRAME (PRIMITIVE_ARITY (primitive));
-	break;
-      }
-
-    case RC_PCOMB2_APPLY:
-      END_SUBPROBLEM ();
-      PUSH_VAL ();		/* Value of arg. 1 */
-      Finished_Eventual_Pushing (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      SET_EXP (MEMORY_REF (GET_EXP, PCOMB2_FN_SLOT));
-      goto primitive_internal_apply;
-
-    case RC_PCOMB2_DO_1:
-      POP_ENV ();
-      PUSH_VAL ();		/* Save value of arg. 2 */
-      DO_ANOTHER_THEN (RC_PCOMB2_APPLY, PCOMB2_ARG_1_SLOT);
-
-    case RC_PCOMB3_APPLY:
-      END_SUBPROBLEM ();
-      PUSH_VAL ();		/* Save value of arg. 1 */
-      Finished_Eventual_Pushing (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      SET_EXP (MEMORY_REF (GET_EXP, PCOMB3_FN_SLOT));
-      goto primitive_internal_apply;
-
-    case RC_PCOMB3_DO_1:
-      {
-	SCHEME_OBJECT Temp = (STACK_POP ()); /* Value of arg. 3 */
-	POP_ENV ();
-	STACK_PUSH (Temp);	/* Save arg. 3 again */
-	PUSH_VAL ();		/* Save arg. 2 */
-	DO_ANOTHER_THEN (RC_PCOMB3_APPLY, PCOMB3_ARG_1_SLOT);
-      }
-
-    case RC_PCOMB3_DO_2:
-      SET_ENV (STACK_REF (0));
-      PUSH_VAL ();		/* Save value of arg. 3 */
-      DO_ANOTHER_THEN (RC_PCOMB3_DO_1, PCOMB3_ARG_2_SLOT);
-
     case RC_POP_RETURN_ERROR:
     case RC_RESTORE_VALUE:
       SET_VAL (GET_EXP);
@@ -1286,20 +1157,11 @@ Interpret (int pop_return_p)
       stack_pointer = (STACK_LOCATIVE_OFFSET (stack_pointer, 1));
       break;
 
-    case RC_SEQ_2_DO_2:
+    case RC_EXECUTE_SEQUENCE_FINISH:
       END_SUBPROBLEM ();
       POP_ENV ();
       REDUCES_TO_NTH (SEQUENCE_2);
 
-    case RC_SEQ_3_DO_2:
-      SET_ENV (STACK_REF (0));
-      DO_ANOTHER_THEN (RC_SEQ_3_DO_3, SEQUENCE_2);
-
-    case RC_SEQ_3_DO_3:
-      END_SUBPROBLEM ();
-      POP_ENV ();
-      REDUCES_TO_NTH (SEQUENCE_3);
-
     case RC_SNAP_NEED_THUNK:
       /* Don't snap thunk twice; evaluation of the thunk's body might
 	 have snapped it already.  */
