@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -45,7 +46,7 @@ USA.
 #  include "ntio.h"
 #endif
 
-#if defined(__WIN32__) || defined(__OS2__)
+#if defined(__WIN32__)
 #  define DOS_LIKE_FILENAMES
 #endif
 
@@ -65,6 +66,14 @@ USA.
 #  endif
 #endif
 
+#ifndef QUOTE_CHAR
+#  ifdef DOS_LIKE_FILENAMES
+#    define QUOTE_CHAR (-1)
+#  else
+#    define QUOTE_CHAR '\\'
+#  endif
+#endif
+
 #ifdef DOS_LIKE_FILENAMES
 #  define FILE_ABSOLUTE(filename)			\
      ((((filename) [0]) == SUB_DIRECTORY_DELIMITER)	\
@@ -82,6 +91,7 @@ static const char * option_raw_band;
 static const char * option_raw_heap;
 static const char * option_raw_constant;
 static const char * option_raw_stack;
+static const char * option_raw_prepend;
 
 /* Command-line arguments */
 int option_saved_argc;
@@ -96,6 +106,9 @@ bool option_disable_core_dump;
 bool option_batch_mode;
 bool option_show_version;
 bool option_show_help;
+#ifdef __APPLE__
+  bool option_macosx_application;
+#endif
 
 /* String options */
 const char ** option_library_path = 0;
@@ -107,6 +120,26 @@ unsigned long option_heap_size;
 unsigned long option_constant_size;
 unsigned long option_stack_size;
 
+#ifndef LIBRARY_PATH_VARIABLE
+#  define LIBRARY_PATH_VARIABLE "MITSCHEME_LIBRARY_PATH"
+#endif
+
+#ifndef DEFAULT_LIBRARY_PATH
+#  ifdef DOS_LIKE_FILENAMES
+#    define DEFAULT_LIBRARY_PATH "c:\\local\\mit-scheme"
+#  else
+#    define DEFAULT_LIBRARY_PATH "/usr/local/lib/mit-scheme"
+#  endif
+#endif
+
+#ifndef BAND_VARIABLE
+#  define BAND_VARIABLE "MITSCHEME_BAND"
+#endif
+
+#ifndef DEFAULT_STD_BAND
+#  define DEFAULT_STD_BAND "all.com"
+#endif
+
 void
 print_help (void)
 {
@@ -121,14 +154,14 @@ for the band.\n\
   list of directories that is searched to find various library files,\n\
   such as bands.  If this option is not given, the value of the\n\
   environment variable MITSCHEME_LIBRARY_PATH is used; if that isn't\n\
-  defined, \"/usr/local/lib/mit-scheme\" is used.\n\
+  defined, \"%s\" is used.\n\
 \n\
 --band FILENAME\n\
   Specifies the initial band to be loaded.  Searches for FILENAME in\n\
   the working directory and the library directories, returning the\n\
   full pathname of the first readable file of that name.  If this\n\
   option isn't given, the filename is the value of the environment\n\
-  variable MITSCHEME_BAND, or if that isn't defined, \"runtime.com\"; in\n\
+  variable MITSCHEME_BAND, or if that isn't defined, \"%s\"; in\n\
   these cases the library directories are searched, but not the\n\
   working directory.\n\
 \n\
@@ -178,38 +211,32 @@ for the band.\n\
 --nocore\n\
   Specifies that Scheme should not generate a core dump under any\n\
   circumstances.\n\
+"
+#ifdef __APPLE__
+"\n\
+--macosx-application\n\
+  Specifies that Scheme is running as a Mac OS X application.\n\
+  This option is automatically supplied when the application is\n\
+  launched from an icon, and should not be given under other\n\
+  circumstances.\n\
+"
+#endif /* __APPLE__ */
+"\n\
+--prepend-library DIRNAME\n\
+  Adds DIRNAME to the front of the library search path.  This option\n\
+  takes one value and can be specified once.\n\
 \n\
 Please report bugs to %s.\n\
 \n\
 Additional options may be supported by the band (and described below).\n\
-\n", PACKAGE_BUGREPORT);
+\n", DEFAULT_LIBRARY_PATH, DEFAULT_STD_BAND, PACKAGE_BUGREPORT);
 }
 
-#ifndef LIBRARY_PATH_VARIABLE
-#  define LIBRARY_PATH_VARIABLE "MITSCHEME_LIBRARY_PATH"
-#endif
-
-#ifndef DEFAULT_LIBRARY_PATH
-#  ifdef DOS_LIKE_FILENAMES
-#    define DEFAULT_LIBRARY_PATH "c:\\local\\mit-scheme"
-#  else
-#    define DEFAULT_LIBRARY_PATH "/usr/local/lib/mit-scheme"
-#  endif
-#endif
-
-#ifndef BAND_VARIABLE
-#  define BAND_VARIABLE "MITSCHEME_BAND"
-#endif
-
-#ifndef DEFAULT_STD_BAND
-#  define DEFAULT_STD_BAND "all.com"
-#endif
-
 #ifndef DEFAULT_HEAP_SIZE
 #  if SIZEOF_UNSIGNED_LONG == 8
 #    define DEFAULT_HEAP_SIZE 16384
 #  else
-#    define DEFAULT_HEAP_SIZE 4096
+#    define DEFAULT_HEAP_SIZE 3072
 #  endif
 #endif
 
@@ -226,7 +253,7 @@ Additional options may be supported by the band (and described below).\n\
 #endif
 
 #ifndef DEFAULT_STACK_SIZE
-#  define DEFAULT_STACK_SIZE 128
+#  define DEFAULT_STACK_SIZE 1024
 #endif
 
 #ifndef STACK_SIZE_VARIABLE
@@ -270,13 +297,93 @@ static char *
 string_copy (const char * s)
 {
   char * result = (OS_malloc ((strlen (s)) + 1));
-  {
-    const char * s1 = s;
-    char * s2 = result;
-    while (((*s2++) = (*s1++)) != '\0') ;
-  }
+  strcpy (result, s);
   return (result);
 }
+
+static char *
+string_copy_limited (const char * s, const char * e)
+{
+  unsigned int n_chars = (e - s);
+  char * result = (OS_malloc (n_chars + 1));
+  strncpy (result, s, n_chars);
+  result[n_chars] = '\0';
+  return (result);
+}
+
+static bool
+must_quote_char_p (int c)
+{
+  return ((c == QUOTE_CHAR) || (c == PATH_DELIMITER));
+}
+
+static unsigned int
+strlen_after_quoting (const char * s)
+{
+  const char * scan = s;
+  unsigned int n_chars = 0;
+  while (true)
+    {
+      int c = (*scan++);
+      if (c == '\0')
+	return n_chars;
+      if (must_quote_char_p (c))
+	n_chars += 1;
+      n_chars += 1;
+    }
+}
+
+static char *
+quote_string (const char * s)
+{
+  const char * scan_in = s;
+  char * result = (OS_malloc ((strlen_after_quoting (s)) + 1));
+  char * scan_out = result;
+  while (true)
+    {
+      int c = (*scan_in++);
+      if (c == '\0')
+	break;
+      if (must_quote_char_p (c))
+	(*scan_out++) = QUOTE_CHAR;
+      (*scan_out++) = c;
+    }
+  (*scan_out) = '\0';
+  return result;
+}
+
+static unsigned int
+strlen_after_unquoting (const char * s)
+{
+  const char * scan = s;
+  unsigned int n_chars = 0;
+  while (true)
+    {
+      int c = (*scan++);
+      if (c == QUOTE_CHAR)
+	c = (*scan++);
+      if (c == '\0')
+	return n_chars;
+      n_chars += 1;
+    }
+}
+
+static char *
+unquote_string (const char * s)
+{
+  const char * scan_in = s;
+  char * result = (OS_malloc ((strlen_after_unquoting (s)) + 1));
+  char * scan_out = result;
+  while (true)
+    {
+      int c = (*scan_in++);
+      if (c == QUOTE_CHAR)
+	c = (*scan_in++);
+      (*scan_out++) = c;
+      if (c == '\0')
+	return result;
+    }
+}
 
 struct option_descriptor
 {
@@ -388,8 +495,12 @@ parse_standard_options (int argc, const char ** argv)
   option_argument ("help", false, (&option_show_help));
   option_argument ("interactive", false, (&option_force_interactive));
   option_argument ("library", true, (&option_raw_library));
+#ifdef __APPLE__
+  option_argument ("macosx-application", false, (&option_macosx_application));
+#endif
   option_argument ("nocore", false, (&option_disable_core_dump));
   option_argument ("option-summary", false, (&option_summary));
+  option_argument ("prepend-library", true, (&option_raw_prepend));
   option_argument ("quiet", false, (&option_batch_mode));
   option_argument ("silent", false, (&option_batch_mode));
   option_argument ("stack", true, (&option_raw_stack));
@@ -475,6 +586,22 @@ get_wd (void)
   }
 }
 
+static const char *
+find_path_delimiter (const char * s)
+{
+  const char * scan = s;
+  while (true)
+    {
+      int c = (*scan++);
+      if (c == PATH_DELIMITER)
+	return (scan - 1);
+      if (c == QUOTE_CHAR)
+	c = (*scan++);
+      if (c == '\0')
+	return 0;
+    }
+}
+
 static const char **
 parse_path_string (const char * path)
 {
@@ -485,50 +612,44 @@ parse_path_string (const char * path)
      that before the scheme process gets too big.  */
   const char * wd = (get_wd ());
   unsigned int lwd = (strlen (wd));
-  while (1)
+  while (true)
     {
-      const char * scan = start;
-      const char * end;
-      while (1)
+      const char * delim = (find_path_delimiter (start));
+      char * unquoted;
+      char * end;
+      char * element;
+      if (delim != 0)
 	{
-	  int c = (*scan++);
-	  if ((c == '\0') || (c == PATH_DELIMITER))
-	    {
-	      end = (scan - 1);
-	      break;
-	    }
+	  const char * copy = (string_copy_limited (start, delim));
+	  unquoted = (unquote_string (copy));
+	  xfree (copy);
 	}
-      if ((start < end) && ((* (end - 1)) == SUB_DIRECTORY_DELIMITER))
-	end -= 1;
-      if (end == start)
-	obstack_ptr_grow ((&scratch_obstack), (string_copy (wd)));
+      else
+	unquoted = (unquote_string (start));
+
+      end = (unquoted + (strlen (unquoted)));
+      if ((end > unquoted) && ((* (end - 1)) == SUB_DIRECTORY_DELIMITER))
+	(*--end) = '\0';
+      if (FILE_ABSOLUTE (unquoted))
+	element = unquoted;
       else
 	{
-	  int absolute = (FILE_ABSOLUTE (start));
-	  {
-	    char * element
-	      = (OS_malloc ((absolute ? 0 : (lwd + 1)) + (end - start) + 1));
-	    char * scan_element = element;
-	    if (!absolute)
-	      {
-		const char * s = wd;
-		const char * e = (wd + lwd);
-		while (s < e)
-		  (*scan_element++) = (*s++);
-		(*scan_element++) = SUB_DIRECTORY_DELIMITER;
-	      }
+	  if ((*unquoted) == '\0')
+	    element = (string_copy (wd));
+	  else
 	    {
-	      const char * s = start;
-	      while (s < end)
-		(*scan_element++) = (*s++);
+	      element = (OS_malloc (lwd + (strlen (unquoted)) + 2));
+	      strcpy (element, wd);
+	      end = (element + lwd);
+	      (*end++) = SUB_DIRECTORY_DELIMITER;
+	      strcpy (end, unquoted);
 	    }
-	    (*scan_element) = '\0';
-	    obstack_ptr_grow ((&scratch_obstack), element);
-	  }
+	  xfree (unquoted);
 	}
-      if ((* (scan - 1)) == '\0')
+      obstack_ptr_grow ((&scratch_obstack), element);
+      if (delim == 0)
 	break;
-      start = scan;
+      start = (delim + 1);
     }
   obstack_ptr_grow ((&scratch_obstack), 0);
   if (wd != 0)
@@ -560,6 +681,20 @@ free_parsed_path (const char ** path)
     }
   xfree (path);
 }
+
+static char *
+add_to_library_path (const char * new_dir, const char * library_path)
+{
+  const char * quoted_dir = (quote_string (new_dir));
+  unsigned int quoted_dir_len = (strlen (quoted_dir));
+  char * result = (OS_malloc (quoted_dir_len + (strlen (library_path)) + 2));
+  char * end = (result + quoted_dir_len);
+  strcpy (result, quoted_dir);
+  (*end++) = PATH_DELIMITER;
+  strcpy (end, library_path);
+  xfree (quoted_dir);
+  return (result);
+}
 
 const char *
 search_for_library_file (const char * filename)
@@ -776,32 +911,32 @@ read_command_line_options (int argc, const char ** argv)
   bool band_sizes_valid = false;
   unsigned long band_constant_size = 0;
   unsigned long band_heap_size = 0;
+  const char * library_path;
   const char * default_library_path = DEFAULT_LIBRARY_PATH;
 
   parse_standard_options (argc, argv);
+
   if (option_library_path != 0)
     free_parsed_path (option_library_path);
 #ifdef __APPLE__
   const char * main_bundle_path = (macosx_main_bundle_dir ());
   if (main_bundle_path != 0)
     {
-      unsigned int n_chars =
-	((strlen (main_bundle_path))
-	 + (strlen (default_library_path))
-	 + 2);
-      char * new_path = (OS_malloc (n_chars));
-      strcpy (new_path, main_bundle_path);
-      strcat (new_path, ":");
-      strcat (new_path, default_library_path);
+      default_library_path =
+	(add_to_library_path (main_bundle_path, default_library_path));
       xfree (main_bundle_path);
-      default_library_path = new_path;
     }
 #endif
-  option_library_path
-    = (parse_path_string
-       (standard_string_option (option_raw_library,
-				LIBRARY_PATH_VARIABLE,
-				default_library_path)));
+  library_path = standard_string_option (option_raw_library,
+					 LIBRARY_PATH_VARIABLE,
+					 default_library_path);
+  if (option_raw_prepend != NULL)
+    {
+      const char * new_path;
+      new_path = add_to_library_path (option_raw_prepend, library_path);
+      library_path = new_path;
+    }
+  option_library_path = parse_path_string (library_path);
 
   if (option_band_file != 0)
     {
@@ -876,7 +1011,10 @@ read_command_line_options (int argc, const char ** argv)
 				STACK_SIZE_VARIABLE,
 				DEFAULT_STACK_SIZE));
   if (option_show_version)
-    outf_console ("%s", PACKAGE_STRING);
+    {
+      outf_console ("%s\n", PACKAGE_STRING);
+      outf_flush_console ();
+    }
   if (option_show_help)
     print_help ();
   if (option_summary)
