@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -182,7 +182,7 @@ OS_nanotime_since_utc_epoch (struct scheme_nanotime *t)
 #elif defined(HAVE_CLOCK_GETTIME)
 
 void
-OS_nanotime_since_utc_epoch (struct scheme_nanotime t)
+OS_nanotime_since_utc_epoch (struct scheme_nanotime *t)
 {
   struct timespec ts;
   STD_VOID_SYSTEM_CALL
@@ -498,6 +498,39 @@ OS_real_time_clock (void)
 
 #endif /* HAVE_TIMES */
 #endif /* HAVE_GETTIMEOFDAY */
+
+/* current-jiffy and jiffies-per-second */
+
+#ifdef HAVE_TIMES
+
+clock_t
+OS_current_jiffy (void)
+{
+  struct tms buffer;
+  return (UX_times (&buffer));
+}
+
+clock_t
+OS_jiffies_per_second (void)
+{
+  return (UX_SC_CLK_TCK ());
+}
+
+#else /* not HAVE_TIMES */
+
+clock_t
+OS_current_jiffy (void)
+{
+  return (clock ());
+}
+
+clock_t
+OS_jiffies_per_second (void)
+{
+  return (CLOCKS_PER_SEC);
+}
+
+#endif /* HAVE_TIMES */
 
 #ifdef HAVE_SETITIMER
 
