@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -36,11 +36,11 @@ USA.
 (define-guarantee stream-pair "stream pair")
 
 (define (stream-car stream)
-  (guarantee-stream-pair stream 'STREAM-CAR)
+  (guarantee stream-pair? stream 'stream-car)
   (car stream))
 
 (define (stream-cdr stream)
-  (guarantee-stream-pair stream 'STREAM-CDR)
+  (guarantee stream-pair? stream 'stream-cdr)
   (force (cdr stream)))
 
 (define the-empty-stream '())
@@ -60,54 +60,57 @@ USA.
 	(loop (force (cdr stream)) (+ n 1))
 	(begin
 	  (if (not (null? stream))
-	      (error:illegal-stream-element stream 'STREAM-LENGTH 0))
+	      (error:illegal-stream-element stream 'stream-length 0))
 	  n))))
 
 (define (stream-ref stream index)
   (let ((tail (stream-tail stream index)))
     (if (not (stream-pair? tail))
-	(error:bad-range-argument index 'STREAM-REF))
+	(error:bad-range-argument index 'stream-ref))
     (car tail)))
 
 (define (stream-head stream index)
-  (guarantee-exact-nonnegative-integer index 'STREAM-HEAD)
+  (guarantee exact-nonnegative-integer? index 'stream-head)
   (let loop ((stream stream) (index index))
     (if (> index 0)
 	(begin
 	  (if (not (stream-pair? stream))
-	      (error:bad-range-argument index 'STREAM-HEAD))
+	      (error:bad-range-argument index 'stream-head))
 	  (cons (car stream)
 		(loop (force (cdr stream)) (- index 1))))
 	'())))
 
 (define (stream-tail stream index)
-  (guarantee-exact-nonnegative-integer index 'STREAM-TAIL)
+  (guarantee exact-nonnegative-integer? index 'stream-tail)
   (let loop ((stream stream) (index index))
     (if (> index 0)
 	(begin
 	  (if (not (stream-pair? stream))
-	      (error:bad-range-argument index 'STREAM-TAIL))
+	      (error:bad-range-argument index 'stream-tail))
 	  (loop (force (cdr stream)) (- index 1)))
 	stream)))
 
+(define (stream-last stream)
+  (stream-car (stream-last-pair stream)))
+
 (define (stream-last-pair stream)
   (if (not (stream-pair? stream))
       (if (null? stream)
-	  (error:bad-range-argument stream 'STREAM-LAST-PAIR)
-	  (error:illegal-stream-element stream 'STREAM-LAST-PAIR 0)))
+	  (error:bad-range-argument stream 'stream-last-pair)
+	  (error:illegal-stream-element stream 'stream-last-pair 0)))
   (let loop ((stream stream))
     (let ((next (force (cdr stream))))
       (if (stream-pair? next)
 	  (loop next)
 	  (begin
-	    (if (not (null? stream))
-		(error:illegal-stream-element stream 'STREAM-LAST-PAIR 0))
+	    (if (not (null? next))
+		(error:illegal-stream-element stream 'stream-last-pair 0))
 	    stream)))))
 
 (define (stream-map procedure stream . streams)
   (cond ((pair? streams)
 	 (let loop ((streams (cons stream streams)))
-	   (receive (cars cdrs) (split-streams streams 'STREAM-MAP)
+	   (receive (cars cdrs) (split-streams streams 'stream-map)
 	     (if (pair? cars)
 		 (cons-stream (apply procedure cars)
 			      (loop (map force cdrs)))
@@ -120,7 +123,7 @@ USA.
 			    (loop (force (cdr stream))))
 	       (begin
 		 (if (not (null? stream))
-		     (error:illegal-stream-element stream 'STREAM-MAP 1))
+		     (error:illegal-stream-element stream 'stream-map 1))
 		 '()))))
 	((and (procedure? stream)
 	      (or (null? procedure) (stream-pair? procedure)))
@@ -132,7 +135,7 @@ USA.
 (define (stream-for-each procedure stream . streams)
   (if (pair? streams)
       (let loop ((streams (cons stream streams)))
-	(receive (cars cdrs) (split-streams streams 'STREAM-FOR-EACH)
+	(receive (cars cdrs) (split-streams streams 'stream-for-each)
 	  (if (pair? cars)
 	      (begin
 		(apply procedure cars)
@@ -142,11 +145,11 @@ USA.
 	       (procedure (car stream))
 	       (loop (force (cdr stream))))
 	      ((not (null? stream))
-	       (error:illegal-stream-element stream 'STREAM-FOR-EACH 1))))))
+	       (error:illegal-stream-element stream 'stream-for-each 1))))))
 
 (define (split-streams streams operator)
-  (let ((cars (list 'CARS))
-	(cdrs (list 'CDRS)))
+  (let ((cars (list 'cars))
+	(cdrs (list 'cdrs)))
     (let loop ((streams streams) (cars-tail cars) (cdrs-tail cdrs) (n 0))
       (if (pair? streams)
 	  (let ((stream (car streams)))
@@ -170,11 +173,11 @@ USA.
 		 (cons-stream (car s) (loop (force (cdr s))))
 		 (begin
 		   (if (not (null? s))
-		       (error:illegal-stream-element s1 'STREAM-APPEND 0))
+		       (error:illegal-stream-element s1 'stream-append 0))
 		   (force s2)))))))
     (if (pair? streams)
 	(let loop ((streams (cons stream streams)))
-	  (receive (cars cdrs) (split-streams streams 'STREAM-APPEND-MAP)
+	  (receive (cars cdrs) (split-streams streams 'stream-append-map)
 	    (if (pair? cars)
 		(sappend (apply procedure cars)
 			 (delay (loop (map force cdrs))))
@@ -185,7 +188,7 @@ USA.
 		       (delay (loop (force (cdr stream)))))
 	      (begin
 		(if (not (null? stream))
-		    (error:illegal-stream-element stream 'STREAM-APPEND-MAP 1))
+		    (error:illegal-stream-element stream 'stream-append-map 1))
 		'()))))))
 
 (define (stream-append . streams)
@@ -198,7 +201,7 @@ USA.
 			       (inner-loop (force (cdr stream))))
 		  (begin
 		    (if (not (null? stream))
-			(error:illegal-stream-element stream 'STREAM-APPEND n))
+			(error:illegal-stream-element stream 'stream-append n))
 		    (outer-loop (cdr streams) (fix:+ n 1)))))
 	    (car streams)))
       '()))
@@ -209,7 +212,7 @@ USA.
 		 (stream-accumulate procedure initial (force (cdr stream))))
       (begin
 	(if (not (null? stream))
-	    (error:illegal-stream-element stream 'STREAM-ACCUMULATE 2))
+	    (error:illegal-stream-element stream 'stream-accumulate 2))
 	initial)))
 
 (define (stream-filter predicate stream)
@@ -220,14 +223,25 @@ USA.
 	  (stream-filter predicate (force (cdr stream))))
       (begin
 	(if (not (null? stream))
-	    (error:illegal-stream-element stream 'STREAM-FILTER 1))
+	    (error:illegal-stream-element stream 'stream-filter 1))
+	'())))
+
+(define (stream-truncate stream predicate)
+  (if (stream-pair? stream)
+      (if (predicate (head stream))
+	  the-empty-stream
+	  (cons-stream (head stream)
+		       (stream-truncate (tail stream) predicate)))
+      (begin
+	(if (not (null? stream))
+	    (error:illegal-stream-element stream 'stream-truncate 1))
 	'())))
 
 (define (stream-write stream #!optional port)
   (let ((port
 	 (if (default-object? port)
 	     (current-output-port)
-	     (guarantee-output-port port 'STREAM-WRITE))))
+	     (guarantee textual-output-port? port 'stream-write))))
     (if (stream-pair? stream)
 	(begin
 	  (write-char #\{ port)
@@ -239,7 +253,7 @@ USA.
 	  (write-char #\} port))
 	(begin
 	  (if (not (null? stream))
-	      (error:illegal-stream-element stream 'STREAM-WRITE 0))
+	      (error:illegal-stream-element stream 'stream-write 0))
 	  (write-string "{}" port)))))
 
 (define (list->stream list)
@@ -247,17 +261,15 @@ USA.
       (cons-stream (car list) (list->stream (cdr list)))
       (begin
 	(if (not (null? list))
-	    (error:not-list list 'LIST->STREAM))
+	    (error:not-a list? list 'list->stream))
 	'())))
 
 (define (stream->list stream)
-  (if (stream-pair? stream)
-      (cons (car stream)
-	    (stream->list (force (cdr stream))))
-      (begin
-	(if (not (null? stream))
-	    (error:illegal-stream-element stream 'STREAM->LIST 0))
-	'())))
+  (let loop ((s stream)
+	     (elements '()))
+    (cond ((stream-pair? s) (loop (tail s) (cons (head s) elements)))
+	  ((null? s) (reverse elements))
+	  (else (error:illegal-stream-element s 'stream->list 0)))))
 
 (define prime-numbers-stream)
 
@@ -267,7 +279,7 @@ USA.
    (letrec
        ((primes (cons-stream 3 (fixnum-filter 5)))
 	(fixnum-filter
-	 (let ((limit (fix:- (largest-fixnum) 2)))
+	 (let ((limit (fix:- (fix:largest-value) 2)))
 	   (lambda (n)
 	     (if (fix:<= n limit)
 		 (let loop ((ps primes))
@@ -302,23 +314,23 @@ USA.
 
 (define (initialize-conditions!)
   (set! condition-type:illegal-stream-element
-	(make-condition-type 'ILLEGAL-STREAM-ELEMENT
+	(make-condition-type 'illegal-stream-element
 	    condition-type:wrong-type-argument
 	    '()
 	  (lambda (condition port)
 	    (write-string "The object " port)
-	    (write (access-condition condition 'DATUM) port)
+	    (write (access-condition condition 'datum) port)
 	    (write-string ", passed as the " port)
 	    (write-string (ordinal-number-string
-			   (+ (access-condition condition 'OPERAND) 1))
+			   (+ (access-condition condition 'operand) 1))
 			  port)
 	    (write-string " argument to " port)
-	    (write-operator (access-condition condition 'OPERATOR) port)
+	    (write-operator (access-condition condition 'operator) port)
 	    (write-string ", is not a stream." port))))
   (set! error:illegal-stream-element
 	(let ((signaller
 	       (condition-signaller condition-type:illegal-stream-element
-				    '(TYPE DATUM OPERATOR OPERAND)
+				    '(type datum operator operand)
 				    standard-error-handler)))
 	  (named-lambda (error:illegal-stream-element stream operator operand)
 	    (signaller "stream" stream operator operand))))
